@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2015. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2016. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -289,7 +289,7 @@ getifaddrs(Socket) ->
 -spec getifaddrs() -> {ok, Iflist} | {error, posix()} when
       Iflist :: [{Ifname,[Ifopt]}],
       Ifname :: string(),
-      Ifopt :: {flag,[Flag]} | {addr,Addr} | {netmask,Netmask}
+      Ifopt :: {flags,[Flag]} | {addr,Addr} | {netmask,Netmask}
              | {broadaddr,Broadaddr} | {dstaddr,Dstaddr}
              | {hwaddr,Hwaddr},
       Flag :: up | broadcast | loopback | pointtopoint
@@ -671,7 +671,7 @@ stats() ->
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 connect_options() ->
     [tos, priority, reuseaddr, keepalive, linger, sndbuf, recbuf, nodelay,
-     header, active, packet, packet_size, buffer, mode, deliver,
+     header, active, packet, packet_size, buffer, mode, deliver, line_delimiter,
      exit_on_close, high_watermark, low_watermark, high_msgq_watermark,
      low_msgq_watermark, send_timeout, send_timeout_close, delay_send, raw,
      show_econnreset].
@@ -692,6 +692,7 @@ connect_options(Opts, Family) ->
     case con_opt(Opts, BaseOpts, connect_options()) of
 	{ok, R} ->
 	    {ok, R#connect_opts {
+		   opts = lists:reverse(R#connect_opts.opts),
 		   ifaddr = translate_ip(R#connect_opts.ifaddr, Family)
 		  }};
 	Error -> Error	    
@@ -721,6 +722,8 @@ con_opt([Opt | Opts], #connect_opts{} = R, As) ->
         {active,N} when is_integer(N), N < 32768, N >= -32768 ->
             NOpts = lists:keydelete(active, 1, R#connect_opts.opts),
             con_opt(Opts, R#connect_opts { opts = [{active,N}|NOpts] }, As);
+	{line_delimiter,C} when is_integer(C), C >= 0, C =< 255 ->
+	    con_add(line_delimiter, C, R, Opts, As);
 	{Name,Val} when is_atom(Name) -> con_add(Name, Val, R, Opts, As);
 	_ -> {error, badarg}
     end;
@@ -760,6 +763,7 @@ listen_options(Opts, Family) ->
     case list_opt(Opts, BaseOpts, listen_options()) of
 	{ok, R} ->
 	    {ok, R#listen_opts {
+		   opts = lists:reverse(R#listen_opts.opts),
 		   ifaddr = translate_ip(R#listen_opts.ifaddr, Family)
 		  }};
 	Error -> Error
@@ -818,6 +822,7 @@ udp_options(Opts, Family) ->
     case udp_opt(Opts, #udp_opts { }, udp_options()) of
 	{ok, R} ->
 	    {ok, R#udp_opts {
+		   opts = lists:reverse(R#udp_opts.opts),
 		   ifaddr = translate_ip(R#udp_opts.ifaddr, Family)
 		  }};
 	Error -> Error
@@ -891,9 +896,12 @@ sctp_options() ->
 sctp_options(Opts, Mod)  ->
     case sctp_opt(Opts, Mod, #sctp_opts{}, sctp_options()) of
 	{ok,#sctp_opts{ifaddr=undefined}=SO} -> 
-	    {ok,SO#sctp_opts{ifaddr=Mod:translate_ip(?SCTP_DEF_IFADDR)}};
-	{ok,_}=OK ->
-	    OK;
+	    {ok,
+	     SO#sctp_opts{
+	       opts=lists:reverse(SO#sctp_opts.opts),
+	       ifaddr=Mod:translate_ip(?SCTP_DEF_IFADDR)}};
+	{ok,SO} ->
+	    {ok,SO#sctp_opts{opts=lists:reverse(SO#sctp_opts.opts)}};
 	Error -> Error
     end.
 
@@ -965,6 +973,8 @@ add_opt(Name, Val, Opts, As) ->
     case lists:member(Name, As) of
 	true ->
 	    case prim_inet:is_sockopt_val(Name, Val) of
+		true when Name =:= raw ->
+		    {ok, [{Name,Val} | Opts]};
 		true ->
 		    Opts1 = lists:keydelete(Name, 1, Opts),
 		    {ok, [{Name,Val} | Opts1]};
